#!/usr/bin/env python
import webob

def app(environ, start_response):
    request = webob.Request(environ)
    if request.path == '/redirect':
        start_response("301 MOVED", [("Location", "/foo")])
        return "301 MOVED"
    elif request.path == '/needs_auth':
        auth = request.headers.get('Authorization')
        if auth and auth.startswith("Basic"):
            user,passwd = auth.split()[-1].decode("base64").split(":")
        else:
            user = None

        if user != 'john':
            start_response("401 Unauthorized", [('WWW-Authenticate', "Basic realm=\"default\"")])
            return "401 Unauthorized"

    start_response("200 OK", [("Content-Type", "text/plain")])
    retval = ["Path: %s" % request.path]
    keys = request.params.keys()
    keys.sort()
    for k in keys:
        v = request.params[k]
        if hasattr(v, 'file'):
            v = v.file.read()
        retval.append("%s: %s" % (k, v))
    return "\n".join(retval)

if __name__ == '__main__':
    import sys
    from paste.httpserver import serve
    port = int(sys.argv[1])
    if len(sys.argv) == 3 and sys.argv[2] == "ssl":
        ssl_pem = "*"
    else:
        ssl_pem = None

    try:
        serve(app, "localhost", port, ssl_pem=ssl_pem)
    except KeyboardInterrupt:
        pass



# -*- coding: utf-8 -*-
from unittest import TestCase
import httplib
import poster
import urllib2, urllib
import threading, time, signal
import sys
import os
import subprocess
import tempfile

port = 5123

class TestStreaming(TestCase):
    disable_https = True
    def setUp(self):
        self.opener = poster.streaminghttp.register_openers()
        if self.disable_https:
            # Disable HTTPS support for these tests to excercise the non-https code
            # HTTPS is tested in test_streaming_https.py
            if hasattr(httplib, "HTTPS"):
                self.https = getattr(httplib, "HTTPS")
                delattr(httplib, "HTTPS")
                reload(poster.streaminghttp)
            else:
                self.https = None
        else:
            self.https = None

        # Hard code to python2.6 for now, since python2.7 can't run the test server reliably
        cmd = ["python2.6", os.path.join(os.path.dirname(__file__), 'test_server.py'), str(port)]
        if not self.disable_https:
            cmd.append("ssl")
        null = open(os.devnull, "w")
        self.server_output = tempfile.TemporaryFile()
        self.server_proc = None
        try:
            self.server_proc = subprocess.Popen(cmd, stdout=self.server_output, stderr=self.server_output, close_fds=True)
            for i in range(20):
                try:
                    if self.disable_https:
                        urllib2.urlopen("http://localhost:%i/" % port).read()
                    else:
                        urllib2.urlopen("https://localhost:%i/" % port).read()
                    time.sleep(0.1)
                    break
                except:
                    #import traceback
                    #traceback.print_exc()
                    time.sleep(0.1)
            else:
                self.server_output.seek(0)
                print self.server_output.read()
                raise OSError("Error starting server")
        except:
            if self.server_proc:
                os.kill(self.server_proc.pid, signal.SIGINT)
                self.server_proc.wait()
            raise

    def tearDown(self):
        if self.https:
            setattr(httplib, "HTTPS", self.https)

        os.kill(self.server_proc.pid, signal.SIGINT)
        self.server_proc.wait()
        self.server_output.seek(0)
        print self.server_output.read()

    def _open(self, url, params=None, headers=None):
        try:
            if headers is None:
                headers = {}
            req = urllib2.Request("http://localhost:%i/%s" % (port, url), params,
                    headers)
            return urllib2.urlopen(req).read()
        except:
            self._opened = False
            raise

    def test_basic(self):
        response = self._open("testing123")

        self.assertEqual(response, "Path: /testing123")

    def test_basic2(self):
        response = self._open("testing?foo=bar")

        self.assertEqual(response, "Path: /testing\nfoo: bar")

    def test_nonstream_uploadfile(self):
        datagen, headers = poster.encode.multipart_encode([
            poster.encode.MultipartParam.from_file("file", __file__),
            poster.encode.MultipartParam("foo", "bar")])

        data = "".join(datagen)

        response = self._open("upload", data, headers)
        self.assertEqual(response, "Path: /upload\nfile: %s\nfoo: bar" % open(__file__).read())

    def test_stream_upload_generator(self):
        datagen, headers = poster.encode.multipart_encode([
            poster.encode.MultipartParam.from_file("file", __file__),
            poster.encode.MultipartParam("foo", "bar")])

        response = self._open("upload", datagen, headers)
        self.assertEqual(response, "Path: /upload\nfile: %s\nfoo: bar" % open(__file__).read())

    def test_stream_upload_file(self):
        data = open("poster/__init__.py")
        headers = {"Content-Length": str(os.path.getsize("poster/__init__.py"))}

        response = self._open("upload", data, headers)
        self.assertEquals(response, "Path: /upload\n%s" % open("poster/__init__.py").read().replace(" = ", " :  "))

    def test_stream_upload_file_no_len(self):
        data = open(__file__)
        self.assertRaises(ValueError, self._open, "upload", data, {})

    def test_stream_upload_generator_no_len(self):
        def data():
            yield ""

        self.assertRaises(ValueError, self._open, "upload", data(), {})

    def test_redirect(self):
        response = self._open("redirect")

        self.assertEqual(response, "Path: /foo")

    def test_login(self):
        password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
        password_manager.add_password(
                None, "http://localhost:%i/needs_auth" % port, 'john', 'secret'
        )

        auth_handler = urllib2.HTTPBasicAuthHandler(password_manager)
        auth_handler.handler_order = 0

        self.opener.add_handler(auth_handler)

        data = open("poster/__init__.py")
        headers = {"Content-Length": str(os.path.getsize("poster/__init__.py"))}

        response = self._open("needs_auth", data, headers)
        self.assertEqual(response, "Path: /needs_auth\n%s" % open("poster/__init__.py").read().replace(" = ", " :  "))

class TestStreamingHTTPS(TestStreaming):
    disable_https = False
    def _open(self, url, params=None, headers=None):
        try:
            if headers is None:
                headers = {}
            req = urllib2.Request("https://localhost:%i/%s" % (port, url), params,
                    headers)
            return urllib2.urlopen(req).read()
        except:
            self._opened = False
            raise



# -*- coding: utf-8 -*-
from unittest import TestCase
import mimetypes
import poster.encode
import StringIO
import sys

def unix2dos(s):
    return s.replace("\n", "\r\n")

class TestEncode_String(TestCase):
    def test_simple(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo"
Content-Type: text/plain; charset=utf-8

bar
""")
        self.assertEqual(expected,
                poster.encode.encode_string("XXXXXXXXX", "foo", "bar"))

    def test_quote_name_space(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo baz"
Content-Type: text/plain; charset=utf-8

bar
""")
        self.assertEqual(expected,
                poster.encode.encode_string("XXXXXXXXX", "foo baz", "bar"))

    def test_quote_name_phparray(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="files[]"
Content-Type: text/plain; charset=utf-8

bar
""")
        self.assertEqual(expected,
                poster.encode.encode_string("XXXXXXXXX", "files[]", "bar"))

    def test_quote_unicode_name(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="=?utf-8?b?4piD?="
Content-Type: text/plain; charset=utf-8

bar
""")
        self.assertEqual(expected,
                poster.encode.encode_string("XXXXXXXXX", u"\N{SNOWMAN}", "bar"))

    def test_quote_value(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo"
Content-Type: text/plain; charset=utf-8

bar baz@bat
""")
        self.assertEqual(expected,
                poster.encode.encode_string("XXXXXXXXX", "foo", "bar baz@bat"))

    def test_boundary(self):
        expected = unix2dos("""--ABC+DEF
Content-Disposition: form-data; name="foo"
Content-Type: text/plain; charset=utf-8

bar
""")
        self.assertEqual(expected,
                poster.encode.encode_string("ABC DEF", "foo", "bar"))

    def test_unicode(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo"
Content-Type: text/plain; charset=utf-8

b\xc3\xa1r
""")
        self.assertEqual(expected,
                poster.encode.encode_string("XXXXXXXXX", "foo", u"bár"))


class TestEncode_File(TestCase):
    def test_simple(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo"
Content-Type: text/plain; charset=utf-8

""")
        self.assertEqual(expected,
                poster.encode.encode_file_header("XXXXXXXXX", "foo", 42))

    def test_content_type(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo"
Content-Type: text/html

""")
        self.assertEqual(expected,
                poster.encode.encode_file_header("XXXXXXXXX", "foo", 42, filetype="text/html"))

    def test_filename_simple(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo"; filename="test.txt"
Content-Type: text/plain; charset=utf-8

""")
        self.assertEqual(expected,
                poster.encode.encode_file_header("XXXXXXXXX", "foo", 42,
                    "test.txt"))

    def test_quote_filename(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo"; filename="test file.txt"
Content-Type: text/plain; charset=utf-8

""")
        self.assertEqual(expected,
                poster.encode.encode_file_header("XXXXXXXXX", "foo", 42,
                    "test file.txt"))

        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo"; filename="test\\"file.txt"
Content-Type: text/plain; charset=utf-8

""")
        self.assertEqual(expected,
                poster.encode.encode_file_header("XXXXXXXXX", "foo", 42,
                    "test\"file.txt"))

    def test_unicode_filename(self):
        expected = unix2dos("""--XXXXXXXXX
Content-Disposition: form-data; name="foo"; filename="&#9731;.txt"
Content-Type: text/plain; charset=utf-8

""")
        self.assertEqual(expected,
                poster.encode.encode_file_header("XXXXXXXXX", "foo", 42,
                    u"\N{SNOWMAN}.txt"))

class TestEncodeAndQuote(TestCase):
    def test(self):
        self.assertEqual("foo+bar", poster.encode.encode_and_quote("foo bar"))
        self.assertEqual("foo%40bar", poster.encode.encode_and_quote("foo@bar"))
        self.assertEqual("%28%C2%A9%29+2008",
                poster.encode.encode_and_quote(u"(©) 2008"))

class TestMultiparam(TestCase):
    def test_from_params(self):
        fp = open("tests/test_encode.py")
        expected = [poster.encode.MultipartParam("foo", "bar"),
                    poster.encode.MultipartParam("baz", fileobj=fp,
                        filename=fp.name,
                        filetype=mimetypes.guess_type(fp.name)[0])]

        self.assertEqual(poster.encode.MultipartParam.from_params(
            [("foo", "bar"), ("baz", fp)]), expected)

        self.assertEqual(poster.encode.MultipartParam.from_params(
            (("foo", "bar"), ("baz", fp))), expected)

        self.assertEqual(poster.encode.MultipartParam.from_params(
            {"foo": "bar", "baz": fp}), expected)

        self.assertEqual(poster.encode.MultipartParam.from_params(
            [expected[0], expected[1]]), expected)

    def test_from_params_dict(self):

        p = poster.encode.MultipartParam('file', fileobj=open("tests/test_encode.py"))
        params = {"foo": "bar", "file": p}

        expected = [poster.encode.MultipartParam("foo", "bar"), p]
        retval = poster.encode.MultipartParam.from_params(params)

        expected.sort()
        retval.sort()

        self.assertEqual(retval, expected)

    def test_from_params_assertion(self):
        p = poster.encode.MultipartParam('file', fileobj=open("tests/test_encode.py"))
        params = {"foo": "bar", "baz": p}

        self.assertRaises(AssertionError, poster.encode.MultipartParam.from_params,
                params)

    def test_simple(self):
        p = poster.encode.MultipartParam("foo", "bar")
        boundary = "XYZXYZXYZ"
        expected = unix2dos("""--XYZXYZXYZ
Content-Disposition: form-data; name="foo"
Content-Type: text/plain; charset=utf-8

bar
--XYZXYZXYZ--
""")
        self.assertEqual(p.encode(boundary), expected[:-len(boundary)-6])
        self.assertEqual(p.get_size(boundary), len(expected)-len(boundary)-6)
        self.assertEqual(poster.encode.get_body_size([p], boundary),
                len(expected))
        self.assertEqual(poster.encode.get_headers([p], boundary),
                {'Content-Length': str(len(expected)),
                 'Content-Type': 'multipart/form-data; boundary=%s' % boundary})

        datagen, headers = poster.encode.multipart_encode([p], boundary)
        self.assertEqual(headers, {'Content-Length': str(len(expected)),
                 'Content-Type': 'multipart/form-data; boundary=%s' % boundary})
        self.assertEqual("".join(datagen), expected)

    def test_multiple_keys(self):
        params = poster.encode.MultipartParam.from_params(
                [("key", "value1"), ("key", "value2")])
        boundary = "XYZXYZXYZ"
        datagen, headers = poster.encode.multipart_encode(params, boundary)
        encoded = "".join(datagen)

        expected = unix2dos("""--XYZXYZXYZ
Content-Disposition: form-data; name="key"
Content-Type: text/plain; charset=utf-8

value1
--XYZXYZXYZ
Content-Disposition: form-data; name="key"
Content-Type: text/plain; charset=utf-8

value2
--XYZXYZXYZ--
""")
        self.assertEqual(encoded, expected)


    def test_stringio(self):
        fp = StringIO.StringIO("file data")
        params = poster.encode.MultipartParam.from_params( [("foo", fp)] )
        boundary = "XYZXYZXYZ"
        datagen, headers = poster.encode.multipart_encode(params, boundary)
        encoded = "".join(datagen)

        expected = unix2dos("""--XYZXYZXYZ
Content-Disposition: form-data; name="foo"
Content-Type: text/plain; charset=utf-8

file data
--XYZXYZXYZ--
""")
        self.assertEqual(encoded, expected)

    def test_reset_string(self):
        p = poster.encode.MultipartParam("foo", "bar")
        boundary = "XYZXYZXYZ"

        datagen, headers = poster.encode.multipart_encode([p], boundary)

        expected = unix2dos("""--XYZXYZXYZ
Content-Disposition: form-data; name="foo"
Content-Type: text/plain; charset=utf-8

bar
--XYZXYZXYZ--
""")

        self.assertEquals("".join(datagen), expected)
        datagen.reset()
        self.assertEquals("".join(datagen), expected)

    def test_reset_multiple_keys(self):
        params = poster.encode.MultipartParam.from_params(
                [("key", "value1"), ("key", "value2")])
        boundary = "XYZXYZXYZ"
        datagen, headers = poster.encode.multipart_encode(params, boundary)
        expected = unix2dos("""--XYZXYZXYZ
Content-Disposition: form-data; name="key"
Content-Type: text/plain; charset=utf-8

value1
--XYZXYZXYZ
Content-Disposition: form-data; name="key"
Content-Type: text/plain; charset=utf-8

value2
--XYZXYZXYZ--
""")

        encoded = "".join(datagen)
        self.assertEqual(encoded, expected)
        datagen.reset()
        encoded = "".join(datagen)
        self.assertEqual(encoded, expected)

    def test_reset_file(self):
        fp = StringIO.StringIO("file data")
        params = poster.encode.MultipartParam.from_params( [("foo", fp)] )
        boundary = "XYZXYZXYZ"
        datagen, headers = poster.encode.multipart_encode(params, boundary)

        expected = unix2dos("""--XYZXYZXYZ
Content-Disposition: form-data; name="foo"
Content-Type: text/plain; charset=utf-8

file data
--XYZXYZXYZ--
""")
        encoded = "".join(datagen)
        self.assertEqual(encoded, expected)
        datagen.reset()
        encoded = "".join(datagen)
        self.assertEqual(encoded, expected)

    def test_MultipartParam_cb(self):
        log = []
        def cb(p, current, total):
            log.append( (p, current, total) )
        p = poster.encode.MultipartParam("foo", "bar", cb=cb)
        boundary = "XYZXYZXYZ"

        datagen, headers = poster.encode.multipart_encode([p], boundary)

        "".join(datagen)

        l = p.get_size(boundary)
        self.assertEquals(log[-1], (p, l, l))

    def test_MultipartParam_file_cb(self):
        log = []
        def cb(p, current, total):
            log.append( (p, current, total) )
        p = poster.encode.MultipartParam("foo", fileobj=open("tests/test_encode.py"),
                cb=cb)
        boundary = poster.encode.gen_boundary()

        list(p.iter_encode(boundary))

        l = p.get_size(boundary)
        self.assertEquals(log[-1], (p, l, l))

    def test_multipart_encode_cb(self):
        log = []
        def cb(p, current, total):
            log.append( (p, current, total) )
        p = poster.encode.MultipartParam("foo", "bar")
        boundary = "XYZXYZXYZ"

        datagen, headers = poster.encode.multipart_encode([p], boundary, cb=cb)

        "".join(datagen)

        l = int(headers['Content-Length'])
        self.assertEquals(log[-1], (None, l, l))

class TestGenBoundary(TestCase):
    def testGenBoundary(self):
        boundary1 = poster.encode.gen_boundary()
        boundary2 = poster.encode.gen_boundary()

        self.assertNotEqual(boundary1, boundary2)
        self.assert_(len(boundary1) > 0)

class TestBackupGenBoundary(TestGenBoundary):
    _orig_import = __import__
    def setUp(self):
        # Make import uuid fail
        def my_import(name, *args, **kwargs):
            if name == 'uuid':
                raise ImportError("Disabled for testing")
            return self._orig_import(name, *args, **kwargs)
        __builtins__['__import__'] = my_import
        reload(poster.encode)

    def tearDown(self):
        __builtins__['__import__'] = self._orig_import
        reload(poster.encode)
